using Luxor
using Colors

const COLOR_ACTIVE = (0.0, 0.0, 0.0)
const COLOR_PREDICTIVE = (0xF5/0xFF, 0x61/0xFF, 0x5C/0xFF)
const COLOR_SELECTED = (0x1B/0xFF, 0xA1/0xFF, 0xEA/0xFF)

const PIX_PER_CELL = 40

n_pix_vert(M::Int) = (M+1)*PIX_PER_CELL
n_pix_horz(N::Int) = (N+1)*PIX_PER_CELL

single_cell_drawing(M::Int, N::Int) = Drawing(n_pix_horz(N), n_pix_vert(M), :png)

function draw_cell_outline!(d::Drawing, M::Int, N::Int)
	sethue(0.0, 0.0, 0.0)
	rect(0.5*PIX_PER_CELL, 0.5*PIX_PER_CELL, N*PIX_PER_CELL, M*PIX_PER_CELL, :stroke)
	return d
end

function draw_cell_states!(d::Drawing, layer::Layer)
	B, M, N = size(layer.basal_dendrite_permanences)
	for m in 1:M
		for n in 1:N
			if layer.A[m,n]
				sethue(COLOR_ACTIVE...)
				circle(n*PIX_PER_CELL, m*PIX_PER_CELL, PIX_PER_CELL/3, :fill)
			elseif layer.Π[m,n]
				sethue(COLOR_PREDICTIVE...)
				circle(n*PIX_PER_CELL, m*PIX_PER_CELL, PIX_PER_CELL/3, :fill)
			end
		end
	end
	return d
end

function draw_input!(d::Drawing, M::Int, ff::BitVector)
	N = length(ff)
	for (n, column_activated) in enumerate(ff)
		if column_activated
			sethue(COLOR_ACTIVE...)
		else
			sethue(0.95, 0.95, 0.95)
		end
		circle(n*PIX_PER_CELL, (M+1)*PIX_PER_CELL, PIX_PER_CELL/3, :fill)
	end
	return d
end

function draw_layer_activity(layer::Layer)
	B, M, N = size(layer.basal_dendrite_permanences)
	d = single_cell_drawing(M, N)
	draw_cell_outline!(d, M, N)
	draw_cell_states!(d, layer)
	finish()
	return d
end

function draw_layer_activity(layer::Layer, ff::BitVector)
	B, M, N = size(layer.basal_dendrite_permanences)
	d = single_cell_drawing(M, N)
	draw_cell_outline!(d, M, N)
	draw_cell_states!(d, layer)
	draw_input!(d, M, ff)
	finish()
	return d
end

function draw_dendrites!(d::Drawing, layer::Layer, m::Int, n::Int)
	# Draw the cell outline
	draw_cell_outline!(d, M, N)

	# Render a circle around the chosen cell
	sethue(COLOR_SELECTED...)
	cell_center = Point(n*PIX_PER_CELL, m*PIX_PER_CELL)
	circle(cell_center, PIX_PER_CELL/2.5, :stroke)

	# Render each dendrite, picking a random color.
	# All active synapses are drawn in that color with higher opacity
	# All inactive synapses are drawn in that color with lower opacity
	gsave()
	for b in 1:B
		hue = b * 360 / B
		rgb = convert(RGB, HSL(hue, 0.75, 0.75))

		U = layer.basal_dendrite_usedness[b,m,n]

		for m′ in 1:M
			for n′ in 1:N
				if U[m′,n′]
					is_synapse_active = layer.A[m′, n′]
					setline(is_synapse_active ? 5 : 3) # thickness
					setcolor(rgb.r, rgb.g, rgb.b, is_synapse_active ? 0.75 : 0.25)
					line(cell_center, Point(n′*PIX_PER_CELL, m′*PIX_PER_CELL), :stroke)
				end
			end
		end
	end
	grestore()

	# Render the states on top
	draw_cell_states!(d, layer)

	return d
end

function draw_dendrites(layer::Layer, m::Int, n::Int)
	B, M, N = size(layer.basal_dendrite_permanences)
	d = single_cell_drawing(M, N)
	draw_dendrites!(d, layer, m, n)
	finish()
	return d
end